/********************************************************************
*  Employee.java       
*
*  Models a basic Employee object for record keeping of the employee's
*  data on a weekly basis.
*
*  Author: Francis Nguyen
*
*  @Version: 1/27/2007
********************************************************************/

import java.text.NumberFormat;

public class Employee
{
   private String name;
   private int idNum;
   private int hours;
   private double payRate;

   //-----------------------------------------------------------------
   //  Constructor #1: Creates a new employee with the specified information.
   //-----------------------------------------------------------------
   public Employee (String newName, int newIdNum, int newHours, double newPayRate)
   {
      name = newName;
      idNum = newIdNum;
      hours = newHours;
      payRate = newPayRate;
   }

   //-----------------------------------------------------------------
   //  Constructor #2: Creates a new employee with the specified information.
   //-----------------------------------------------------------------
   public Employee (String newName, int newIdNum)
   {
      name = newName;
      idNum = newIdNum;
   }

   //-----------------------------------------------------------------
   //  Computes and returns the payment for this hourly employee.
   //-----------------------------------------------------------------
   public double getWeeklyPay ()
   {
      double weeklyPay;

      if (hours > 40)
         weeklyPay = hours * payRate + (hours - 40) * (payRate * 1.2);
      else
         weeklyPay = hours * payRate;

      return weeklyPay;
   }

   //-----------------------------------------------------------------
   //  Returns the Employee's name
   //-----------------------------------------------------------------
   public String getName ()
   {
      return name;
   } 

   //-----------------------------------------------------------------
   //  Returns the Employee's ID number
   //-----------------------------------------------------------------
   public int getIdNum ()
   {
      return idNum;
   } 

   //-----------------------------------------------------------------
   //  Returns the number of hours worked.
   //-----------------------------------------------------------------
   public int getHours ()
   {
      return hours;
   }

   //-----------------------------------------------------------------
   //  Returns pay per hour.
   //-----------------------------------------------------------------
   public double getPayRate ()
   {
      return payRate;
   } 
 
   //-----------------------------------------------------------------
   //  Updates the Employee's name.
   //-----------------------------------------------------------------
   public void setName (String newName)
   {
      name = newName;
   }

   //-----------------------------------------------------------------
   //  Updates the Employee's ID.
   //-----------------------------------------------------------------
   public void setIdNum (int newIdNum)
   {
       idNum = newIdNum;
   }

   //-----------------------------------------------------------------
   //  Updates the number of hours worked.
   //-----------------------------------------------------------------
   public void setHours (int newHours)
   {
      hours = newHours;
   }

   //-----------------------------------------------------------------
   //  Updates the employee's hourly pay
   //-----------------------------------------------------------------
   public void setPayRate (double newPayRate)
   {
      payRate = newPayRate;
   }

   //-----------------------------------------------------------------
   //  Compare only the employee's ID and name.
   //-----------------------------------------------------------------
   public boolean equals (Employee otherEmployee)
   {
       return (name == otherEmployee.getName() && idNum == otherEmployee.getIdNum());
   }

   //-----------------------------------------------------------------
   //  Returns a string description of this product.
   //-----------------------------------------------------------------
   public String toString ()
   {
      NumberFormat fmt = NumberFormat.getCurrencyInstance();
     
      String description;
   
      description = name + "\t\t" + idNum+ "\t\t";
      description += hours + "\t\t" + fmt.format (payRate);

      return description;
   }
}
