/********************************************************************
*  EmployeeCollection.java     
*
*  Manages an array of Employee objects.
*
*  Author: Francis Nguyen
*
*  @Version: 1/27/2007
********************************************************************/

import java.text.NumberFormat;
import java.io.*;
import java.util.Scanner;

public class EmployeeCollection
{
   private Employee[] collection;
   private int count;

   //-----------------------------------------------------------------
   //  Constructor: Creates an initially empty collection.
   //-----------------------------------------------------------------
   public EmployeeCollection ()
   {
      collection = new Employee[10];
      count = 0;
   }

   //-----------------------------------------------------------------
   //  Adds a employee to the collection, increasing the size of the
   //  collection if necessary.
   //-----------------------------------------------------------------
   public void addEmployee (String name, int idNum, int hours, double payRate)                          
   {
      if (count == collection.length)
         increaseSize();

      if (count == 0)
      {
         if (name.length() >= 5)
         {
            if (idNum >= 0 && idNum <= 99999)
            {
               if (hours >= 0 && hours <= 60)
               {
                  if (payRate >= 8.25 && payRate <= 35.00)
                  {
                     collection[count] = new Employee (name, idNum, hours, payRate);
                     count++;
                  }               
                  else
                     System.out.println ("Employee not added -- invalid pay rate.");
               }               
               else
                  System.out.println ("Employee not added -- invalid hours.");   
            }                       
            else
               System.out.println ("Employee not added -- invalid ID number.");    
         }               
         else
            System.out.println ("Employee not added -- name cannot be shorter than 5 characters long.");  
      }
      else
      {
         int index = findEmployee (idNum);
         if (index == -1)
         {
            if (name.length() >= 5)
            {
               if (idNum >= 0 && idNum <= 99999)
               {
                  if (hours >= 0 && hours <= 60)
                  {
                     if (payRate >= 8.25 && payRate <= 35.00)
                     {
                        collection[count] = new Employee (name, idNum, hours, payRate);
                         count++;
                     }               
                     else
                        System.out.println ("Amount is $0.00 -- invalid pay rate.");
                  }               
                  else
                     System.out.println ("Employee not added -- invalid hours.");   
               }                       
               else
                  System.out.println ("Employee not added -- invalid ID number.");    
            }               
            else
               System.out.println ("Employee not added -- name cannot be shorter than 5 characters long.");  
         }
         else
            System.out.println ("Employee not added -- it already exists in the array.");
      }
   }

   //-----------------------------------------------------------------
   //  Adds a employee to the collection, increasing the size of the
   //  collection if necessary.
   //-----------------------------------------------------------------
   public void addEmployee (String name, int idNum)                          
   {
      int hours = 40;
      double payRate = 8.25;

      if (count == collection.length)
         increaseSize();

      if (count == 0)
      {
         if (name.length() >= 5)
         {
            if (idNum >= 0 && idNum <= 99999)
            {
               collection[count] = new Employee (name, idNum);
               count++;
            }                                 
            else
               System.out.println ("Employee not added -- invalid ID number.");    
         }               
         else
            System.out.println ("Employee not added -- name cannot be shorter than 5 characters long."); 
      }
      else
      {
         int index = findEmployee (idNum);
         if (index == -1)
         {
            if (name.length() >= 5)
            {
               if (idNum >= 0 && idNum <= 99999)
               {
                  collection[count] = new Employee (name, idNum);
                  count++;
               }                                 
               else
                  System.out.println ("Employee not added -- invalid ID number.");    
            }               
            else
               System.out.println ("Employee not added -- name cannot be shorter than 5 characters long."); 
         }
         else
            System.out.println ("Employee not added -- it already exists in the array.");
      }

   }

   //-----------------------------------------------------------------
   //  Removes the employee from the array, by shifting all employees
   //  below it up one compartment and subtracting one from the
   //  variable named, count.
   //-----------------------------------------------------------------
   public void deleteEmployee (int idNum)
   {
      String userResponse;
      Scanner keyboard = new Scanner(System.in);

      int index = findEmployee (idNum);

      if (index == -1)
        System.out.println ("\nThe employee was not found in the collection.");

      else
        if (index != -1)
        {
           System.out.print ("\nAre you sure you want to delete this employee? ");
           userResponse = keyboard.nextLine();

           if (userResponse.equalsIgnoreCase("yes"))
           {
              for (int i = index; i < count; i++)
              collection[i] = collection[i+1];
              count--; 

              System.out.println ("\nThis employee has been deleted.");
           }
        }

   }

   //-----------------------------------------------------------------
   //  Removes the employee from the array, by shifting all employees
   //  below it up one compartment and subtracting one from the
   //  variable named, count.
   //-----------------------------------------------------------------
   public void deleteEmployee (String name)
   {
      String userResponse;
      Scanner keyboard = new Scanner(System.in);
      
      int index = findEmployee (name);

      if (index == -1)
        System.out.println ("\nThe employee was not found in the collection.");

      else
        if (index != -1)
        {
           System.out.print ("\nAre you sure you want to delete this employee? ");
           userResponse = keyboard.nextLine();

           if (userResponse.equalsIgnoreCase("yes"))
           {
              for (int i = index; i < count; i++)
              collection[i] = collection[i+1];
              count--; 

              System.out.println ("\nThis employee has been deleted.");
           }
        }

   }

   //-----------------------------------------------------------------
   //  Displays the employee's ID and an informative message if the 
   //  employee was not found in the array.
   //-----------------------------------------------------------------
   public void displayEmployee (int idNum)
   {
      NumberFormat fmt = NumberFormat.getCurrencyInstance();
      String report;

      int index = findEmployee (idNum);

      if (index == -1)
        System.out.println ("\nThe employee was not found in the collection.\n");

      else   
         if (index != -1)
         {
            report = "\nName\t\tID Number\tHours Per Week\tPay Per Hour\tAmount\n";
            report += collection[index].toString () + "\t\t" + fmt.format(collection[index].getWeeklyPay()) + "\n";
            System.out.println (report);
         } 

   }

   //-----------------------------------------------------------------
   //  Displays the employee's name and an informative message if the 
   //  employee was not found in the array.
   //-----------------------------------------------------------------
   public void displayEmployee (String name)
   {
      NumberFormat fmt = NumberFormat.getCurrencyInstance();
      String report;

      int index = findEmployee (name);

      if (index == -1)
        System.out.println ("\nThe employee was not found in the collection.\n");

      else
         if (index != -1)
         {
            report = "\nName\t\tID Number\tHours Per Week\tPay Per Hour\tAmount\n";
            report += collection[index].toString () + "\t\t" + fmt.format(collection[index].getWeeklyPay()) + "\n";
            System.out.println (report);
         } 

   }

   //-----------------------------------------------------------------
   //  Computes the average of hours worked by all the employees.
   //-----------------------------------------------------------------
   public void average ()  
   {
      double average; 
      int sum = 0;

      for (int i = 0; i < count; i++)  
         sum += collection[i].getHours ();
      
      average = sum / count;

      System.out.print ("Average of hours worked by all the employees: "); 
      System.out.println (average + " hours\n");  
   }

   //-----------------------------------------------------------------
   //  Lists employees that earned more than a certain amount of money.
   //-----------------------------------------------------------------
   public void lowerLimit ()  
   {
      NumberFormat fmt = NumberFormat.getCurrencyInstance();
      String report;

      double lowerLimit;
      Scanner keyboard = new Scanner(System.in);

        int index = findWeeklyPay();

        if (index != -1)
        {
           System.out.print ("Enter the amount of money used as the lower limit? ");
           lowerLimit = keyboard.nextDouble();

           if (lowerLimit >= 0)
           {
              for (int i = index; i < count; i++)
              {
                 if (collection[i].getWeeklyPay() > lowerLimit)
                 {
                    report = "Name\t\tID Number\tHours Per Week\tPay Per Hour\tAmount\n";
                    report += collection[i].toString () + "\t\t" + fmt.format(collection[i].getWeeklyPay()) + "\n";
                    System.out.println (report);
                 }
              }
           }
           else
              System.out.println ("\nNo negative number");
        }             
   }

   //-----------------------------------------------------------------
   //  Lists employees that earned less than a given amount of money.
   //-----------------------------------------------------------------
   public void upperLimit ()  
   {
      NumberFormat fmt = NumberFormat.getCurrencyInstance();
      String report;

      double upperLimit; 
      Scanner keyboard = new Scanner(System.in);

      int index = findWeeklyPay();

      if (index != -1)
      {
         System.out.print ("Enter the amount of money used as the upper limit? ");
         upperLimit = keyboard.nextDouble();

         if (upperLimit >= 0)
         {
            for (int i = index; i < count; i++)
            {
               if (collection[i].getWeeklyPay() < upperLimit)
               {
                  report = "Name\t\tID Number\tHours Per Week\tPay Per Hour\tAmount\n";
                  report += collection[i].toString () + "\t\t" + fmt.format(collection[i].getWeeklyPay()) + "\n";
                  System.out.println (report);
               }
            }
         }
         else
            System.out.println ("\nNo negative number");
      }               
   }

   //-----------------------------------------------------------------
   //  Lists employees that worked overtime.
   //-----------------------------------------------------------------
   public void overtime ()  
   {
      NumberFormat fmt = NumberFormat.getCurrencyInstance();
      String report;

      for (int i =  0; i < count; i++)
         if (collection[i].getHours () > 40)
         {
            report = "Name\t\tID Number\tHours Per Week\tPay Per Hour\tAmount\n";
            report += collection[i].toString () + "\t\t" + fmt.format(collection[i].getWeeklyPay()) + "\n";
            System.out.println (report);
         }        
   }

   //-----------------------------------------------------------------
   //  Returns the index into the collection array where the amount
   //  was found.  Returns -1 if the employee is not in the array.
   //-----------------------------------------------------------------
   public int findWeeklyPay ()
   {
      int index = -1;

      for (int i = 0; i < count; i++)
      {
         collection[i].getWeeklyPay();  
         index = i;
         i = count;
      }    
      return index;
   }

   //-----------------------------------------------------------------
   //  Returns the index into the collection array where the idNum
   //  was found.  Returns -1 if the employee is not in the array.
   //-----------------------------------------------------------------
   public int findEmployee (int idNum)
   {
      int index = -1;

      for (int i = 0; i < count; i++)
         if (collection[i].getIdNum () == idNum)    
         {
            index = i;
            i = count;
         }    
      return index;
   }

   //-----------------------------------------------------------------
   //  Returns the index into the collection array where the name
   //  was found.  Returns -1 if the employee is not in the array.
   //-----------------------------------------------------------------
   public int findEmployee (String name)
   {
      int index = -1;

      for (int i = 0; i < count; i++)
        if (collection[i].getName().equalsIgnoreCase (name))
        {  
           index = i;
           i = count;
        }  
      return index;
   }

   //-----------------------------------------------------------------
   //  Returns count.
   //-----------------------------------------------------------------
   public int getCount ()
   {
      return count;
   }

   //-----------------------------------------------------------------
   //  Returns collection[i].
   //-----------------------------------------------------------------
   public Employee getEmployee (int i)
   {
      return collection[i];
   }

   //-----------------------------------------------------------------
   //  Returns a report describing the product collection.
   //-----------------------------------------------------------------
   public String toString ()
   {
      NumberFormat fmt = NumberFormat.getCurrencyInstance();

      String report = "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n";
      report += "My Employee Collection\n";

      report += "Number of Employees: " + count + "\n";

      report += "\nEmployee List:\n";

      report += "Name\t\tID Number\tHours Per Week\tPay Per Hour\tAmount\n";

      for (int employee = 0; employee < count; employee++)
         report += collection[employee].toString () + "\t\t" + fmt.format(collection[employee].getWeeklyPay()) + "\n";

      return report;
   }

   //-----------------------------------------------------------------
   //  Increases the capacity of the collection by creating a
   //  larger array and copying the existing collection into it.
   //-----------------------------------------------------------------
   private void increaseSize ()
   {
      Employee[] temp = new Employee[collection.length * 2];

      for (int employee = 0; employee < collection.length; employee++)
         temp[employee] = collection[employee];

      collection = temp;
   }
}
