/********************************************************************
*  ProgramOne.java      
*
*  Reads data necessary to create an array of employee objects,
*  displays a menu and obtains data necessary to process transactions
*  on the array, and creates an updated text file before ending the
*  program.
*
*  Author: Francis Nguyen
*
*  @Version: 1/27/2007
********************************************************************/

import java.io.*;
import java.util.Scanner;
import java.util.InputMismatchException;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

public class ProgramOne
{
   //-----------------------------------------------------------------
   //  Creates a Employee Collection object and adds some employees to it.
   //  Prints reports on the status of the collection.
   //-----------------------------------------------------------------
   public static void main (String[] args) throws FileNotFoundException, IOException
   {
      EmployeeCollection employee = new EmployeeCollection ();
    
      int count = 0;
      String name = null;
      int idNum = 0;
      int hours = 0;
      double payRate = 0.0;
      int option;
      String userResponse;
      String lineIn = null; 
      StringTokenizer st = null;

      Scanner scan = new Scanner (new File ("employee.txt"));
      Scanner keyboard = new Scanner (System.in);

      lineIn = scan.nextLine();

      while (lineIn != null)
      {
         try
         {
            st = new StringTokenizer (lineIn,",");

   	    if (st.countTokens() == 2 || st.countTokens() == 4)
            {
	       name = st.nextToken();
               idNum = Integer.parseInt (st.nextToken().substring(0,5));             

               if (name.length() >= 5) 
               {   
                  if (st.hasMoreTokens() && (idNum >= 0 && idNum <= 99999))
                  {      
	             hours = Integer.parseInt (st.nextToken());
	             payRate = Double.parseDouble (st.nextToken()); 
   
   	             if (hours >= 0 && hours <= 60)
                     {
                        if (payRate >= 8.25 && payRate <= 35.00)
                        {
                           employee.addEmployee (name, idNum, hours, payRate);      
		        } 
                        else
	                   System.out.println ("Invalid record ignored.");  
		     } 
                     else
	                System.out.println ("Invalid record ignored."); 
	          } 
                  else
	          {
                     employee.addEmployee (name, idNum);
                  }                   
	       } 
               else
	          System.out.println ("Invalid record ignored."); 
	    } 
            else
	       System.out.println ("Invalid record ignored."); 
  
         }

         catch (InputMismatchException exception)
         {
            System.out.println ("Invalid record ignored.");
            scan.nextLine();
         }  

         catch (NoSuchElementException exception)
         {
            System.out.println ("Invalid record ignored.");
         }

         catch (StringIndexOutOfBoundsException exception)
         {
            System.out.println ("Invalid record ignored.");
         }

         catch (NumberFormatException exception)
         {
            System.out.println ("Invalid record ignored.");
         }

   	 if (scan.hasNextLine())
	    lineIn = scan.nextLine();
  	 else
	    lineIn = null;

      }
      System.out.println (employee);

      do
      {
         //display menu
         //prompt for user option
         System.out.println ("Option 1: Add Employee");
         System.out.println ("Option 2: Remove an Employee");
         System.out.println ("Option 3: Get the average of hours worked by all the employees");
         System.out.println ("Option 4: List the employees that earned more than a certain amount of money");
         System.out.println ("Option 5: List the employees that worked overtime");
         System.out.println ("Option 6: List employees that earned less than a given amount of money");
         System.out.println ("Option 7: Find a given Employee and retrieve his/her data");
         System.out.println ("Option 8: List all the employees");
         System.out.println ("Option 0: Exit");
         System.out.println (); 
         System.out.print ("Please select one of the following options from the main menu above > ");
         option = keyboard.nextInt ();
         System.out.println ();
            
         switch (option)
         {
	    case 8:	
		//List all the employees
                System.out.println ("\n" + employee);
                keyboard.nextLine ();
		break;

            case 7:
         	//Find a given Employee and retrieve his/her data
		do
 		{                       
                   	//display menu
              	   	//prompt for user option                               
              		System.out.println ();
              		System.out.println ("Option 1: Find the employee by name");
              		System.out.println ("Option 2: Find the employee by ID number");
              		System.out.println ("Option 0: Cancel the operation and go back to the main menu ");
              		System.out.println (); 
              		System.out.print ("Please select one of the following options > ");
              		option = keyboard.nextInt ();
              		System.out.println ();

              		switch (option)
              		{
	          	   case 2:	
				//Look for the employee by ID number
				System.out.print ("Enter the employee's ID number > ");
                        	idNum = keyboard.nextInt ();
                        	employee.displayEmployee (idNum);
                       		keyboard.nextLine ();
                       		break;

                 	   case 1:
				//Look for the employee by name
				System.out.print ("Enter the employee's > ");
				keyboard.nextLine ();
                        	name = keyboard.nextLine ();
                        	employee.displayEmployee (name);
                        	keyboard.nextLine ();
                                break;

               		}  //end of switch  
           		option = 0;

         		System.out.print ("Would you like to select another option? ");
         		userResponse = keyboard.nextLine();
                        System.out.println(); 
		}
      		while (option != 0 || userResponse.equalsIgnoreCase ("yes"));
                break;

            case 6:
                //List employees that earned less than a given amount of money  
                employee.upperLimit ();
                keyboard.nextLine ();
                break;                   

            case 5: 
		//List the employees that worked overtime
                System.out.println ("Employees that worked overtime: ");
                employee.overtime ();
                keyboard.nextLine ();
                break;
      
            case 4:    
		//List the employees that earned more than a certain amount of money
                employee.lowerLimit ();    
                keyboard.nextLine ();
		break;

            case 3:    
		//Get the average of hours worked by all the employees
                employee.average ();
		keyboard.nextLine ();
                break;

	    case 2:
		//Remove an Employee
		do
 		{                       
                   	//display menu
              	   	//prompt for user option                               
              		System.out.println ();
              		System.out.println ("Option 1: Look for the employee by name ");
              		System.out.println ("Option 2: Find the employee by ID number ");
              		System.out.println ("Option 0: Cancel the operation and go back to the main menu ");
              		System.out.println (); 
              		System.out.print ("Please select one of the following options > ");
              		option = keyboard.nextInt ();
              		System.out.println ();

              		switch (option)
              		{
	          	   case 2:	
				//Look for the employee by ID number
				System.out.print ("Enter the employee's ID number? ");
                        	idNum = keyboard.nextInt ();
                        	employee.deleteEmployee (idNum);
                        	System.out.println ("\n" + employee);
                       		keyboard.nextLine ();
                       		break;

                 	   case 1:
				//Look for the employee by name
				System.out.print ("Enter the employee's name? ");
				keyboard.nextLine ();
                        	name = keyboard.nextLine ();
                        	employee.deleteEmployee (name);
                        	System.out.println ("\n" + employee);
                        	keyboard.nextLine ();
                                break;

               		}  //end of switch  
           		option = 0;

         		System.out.print ("Would you like to select another option? ");
         		userResponse = keyboard.nextLine();
                        System.out.println (); 
		}
      		while (option != 0 || userResponse.equalsIgnoreCase ("yes"));
                break;
			
            case 1:      
		//Add employee
                System.out.println ("Enter the data about the new employee: ");
      
                System.out.print ("Name > ");
                keyboard.nextLine ();
                name = keyboard.nextLine (); 

                System.out.print ("ID Number > ");
                idNum = keyboard.nextInt ();

                System.out.print ("Number of hours > ");
                hours = keyboard.nextInt ();
  
                System.out.print ("Pay Per Hour > ");    
                payRate = keyboard.nextDouble ();

                employee.addEmployee (name, idNum, hours, payRate);
                employee.addEmployee (name, idNum);
                System.out.println ("\n" + employee);
		keyboard.nextLine ();
                break;

         }  //end of switch  
         option = 0;
  
         System.out.print ("Would you like to select another option from the main menu? ");
         userResponse = keyboard.nextLine ();

      }
      while (option != 0 || userResponse.equalsIgnoreCase ("yes"));

      FileWriter fw = new FileWriter ("employeeUpdate.doc");
      BufferedWriter bw = new BufferedWriter (fw);
      PrintWriter pw = new PrintWriter (bw);
      int employeeCount = employee.getCount ();
      Employee myEmployee = null;
      for (int i = 0; i < employeeCount; i++)
      {
         myEmployee = employee.getEmployee (i);
         pw.print (myEmployee.getName () + ",");
         pw.print (myEmployee.getIdNum () + ",");
         pw.print (myEmployee.getHours () + ",");
         pw.println (myEmployee.getPayRate () + ",");
      }

      pw.close ();
   }
}